'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.setup = setup;

var _parseInfo = require('./parseInfo');

var _parseInfo2 = _interopRequireDefault(_parseInfo);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Characteristic = void 0;
var Service = void 0;

function setup(homebridge) {
  Characteristic = homebridge.hap.Characteristic; // eslint-disable-line prefer-destructuring
  Service = homebridge.hap.Service; // eslint-disable-line prefer-destructuring
}

var EQ3BLEAccessory = function () {
  function EQ3BLEAccessory(log, device, disableBoostSwitch) {
    _classCallCheck(this, EQ3BLEAccessory);

    this.state = {};
    this.log = log;
    this.name = device.address;
    this.address = device.address;
    this.disableBoostSwitch = disableBoostSwitch;
    this.temperatureDisplayUnits = Characteristic.TemperatureDisplayUnits.CELSIUS;

    this.thermostat = device;

    this.thermostatService = new Service.Thermostat(this.name);
    this.informationService = new Service.AccessoryInformation();
    this.boostService = new Service.Switch(this.name + ' boost mode');

    this.currentTemperature = null;

    this.setupServices();
  }

  _createClass(EQ3BLEAccessory, [{
    key: 'getServices',
    value: function getServices() {
      var services = [this.informationService, this.thermostatService];
      if (!this.disableBoostSwitch) services.push(this.boostService);
      return services;
    }
  }, {
    key: 'setBoost',
    value: function setBoost(boost, callback) {
      var _this = this;

      if (boost === this.state.boost) {
        callback(null);
        return;
      }
      this.state.boost = boost;
      this.log('set boost', this.name, boost);
      this.thermostat.setBoost(boost).then(function () {
        _this.thermostat.getInfo();
        callback(null);
      }).catch(function (err) {
        return callback(err);
      });
    }
  }, {
    key: 'setThermostatState',
    value: function setThermostatState(state) {
      switch (state) {
        case Characteristic.TargetHeatingCoolingState.OFF:
          return this.thermostat.turnOff();
        case Characteristic.TargetHeatingCoolingState.HEAT:
          return this.thermostat.turnOn();
        case Characteristic.TargetHeatingCoolingState.AUTO:
          return this.thermostat.automaticMode();
        case Characteristic.TargetHeatingCoolingState.COOL:
          return this.thermostat.manualMode();
        default:
          throw new Error('Unsupported mode');
      }
    }
  }, {
    key: 'setTargetHeatingCoolingState',
    value: function setTargetHeatingCoolingState(state, callback) {
      var _this2 = this;

      if (state === this.state.targetHeatingCoolingState) {
        callback(null);
        return;
      }
      this.state.targetHeatingCoolingState = state;
      this.log('set state', this.name, state);
      this.setThermostatState(state).then(function () {
        _this2.thermostat.getInfo();
        callback(null);
      }).catch(function (err) {
        return callback(err);
      });
    }
  }, {
    key: 'setTargetTemperature',
    value: function setTargetTemperature(temperature, callback) {
      var _this3 = this;

      if (temperature === this.state.targetTemperature) {
        callback(null);
        return;
      }
      this.state.targetTemperature = temperature;
      this.log('set temperature', this.name, temperature);
      this.thermostat.setTemperature(temperature).then(function () {
        _this3.thermostat.getInfo();
        callback(null);
      }).catch(function (err) {
        return callback(err);
      });
    }
  }, {
    key: 'setupServices',
    value: function setupServices() {
      var _this4 = this;

      this.boostService.setCharacteristic(Characteristic.Name, this.name + ' Boost Mode');

      this.informationService.setCharacteristic(Characteristic.Manufacturer, 'eq-3').setCharacteristic(Characteristic.Model, 'CC-RT-BLE');

      var boostOn = this.boostService.getCharacteristic(Characteristic.On);
      var currHeatingCoolingState = this.thermostatService.getCharacteristic(Characteristic.CurrentHeatingCoolingState);
      var targetHeatingCoolingState = this.thermostatService.getCharacteristic(Characteristic.TargetHeatingCoolingState);
      var currentTemperature = this.thermostatService.getCharacteristic(Characteristic.CurrentTemperature).setProps({
        minValue: -100,
        maxValue: 100
      });
      var targetTemperature = this.thermostatService.getCharacteristic(Characteristic.TargetTemperature).setProps({
        minValue: 4.5,
        maxValue: 30
      });
      var heatingThresholdTemperature = this.thermostatService.getCharacteristic(Characteristic.HeatingThresholdTemperature);

      boostOn.on('get', function (callback) {
        return callback(null, _this4.state.boost);
      }).on('set', this.setBoost.bind(this));

      currHeatingCoolingState.on('get', function (callback) {
        return callback(null, _this4.state.currentHeatingCoolingState);
      });

      targetHeatingCoolingState.on('get', function (callback) {
        return callback(null, _this4.state.targetHeatingCoolingState);
      }).on('set', this.setTargetHeatingCoolingState.bind(this));

      currentTemperature.on('get', function (callback) {
        return callback(null, _this4.state.targetTemperature);
      });

      targetTemperature.on('get', function (callback) {
        return callback(null, _this4.state.targetTemperature);
      }).on('set', this.setTargetTemperature.bind(this));

      heatingThresholdTemperature.on('get', function (callback) {
        return callback(null, _this4.state.targetTemperature);
      });

      this.thermostat.on('update', function (data) {
        _this4.log('update', _this4.thermostat.address, data);
        _this4.state = (0, _parseInfo2.default)(data, Characteristic);
        boostOn.setValue(_this4.state.boost);
        currHeatingCoolingState.setValue(_this4.state.currentHeatingCoolingState);
        targetHeatingCoolingState.setValue(_this4.state.targetHeatingCoolingState);
        targetTemperature.setValue(_this4.state.targetTemperature);
        heatingThresholdTemperature.setValue(_this4.state.targetTemperature);
        currentTemperature.setValue(_this4.state.targetTemperature);
      });

      this.thermostat.getInfo();
      setInterval(function () {
        return _this4.thermostat.getInfo();
      }, 10 * 60 * 1000);
    }
  }]);

  return EQ3BLEAccessory;
}();

exports.default = EQ3BLEAccessory;