'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = parseInfo;

var _constants = require('./constants');

function parseTargetState(_ref) {
  var targetTemperature = _ref.targetTemperature,
      manualMode = _ref.manualMode,
      boost = _ref.boost;

  if (targetTemperature <= 4.5) return _constants.TargetHeatingCoolingState.OFF;
  if (targetTemperature >= 30 || boost) return _constants.TargetHeatingCoolingState.HEAT;
  if (manualMode) return _constants.TargetHeatingCoolingState.COOL;
  return _constants.TargetHeatingCoolingState.AUTO;
}

function parseInfo(data, Characteristic) {
  var boost = data.boost,
      valvePosition = data.valvePosition,
      targetTemperature = data.targetTemperature;

  var info = { boost: boost, targetTemperature: targetTemperature };

  info.currentHeatingCoolingState = (0, _constants.map)(Characteristic, valvePosition ? _constants.CurrentHeatingCoolingState.HEAT : _constants.CurrentHeatingCoolingState.OFF);

  info.targetHeatingCoolingState = (0, _constants.map)(Characteristic, parseTargetState(data));

  return info;
}